searchNodes=[{"ref":"Mix.Tasks.ZigDoc.html","title":"Mix.Tasks.ZigDoc","type":"task","doc":"Used to generate documentation from your zig code and incorporate it into your Elixir documentation. Generally speaking, zig structs will be documented as Elixir modules and zig functions will be incorporated as Elixir functions. If you want to make the zig docgen process override the Elixir mix docs task, add this to your mix.exs: def project do [ ... aliases: [docs: &quot;zig_doc&quot;], ... ] end"},{"ref":"Zigler.html","title":"Zigler","type":"module","doc":"Inline NIF support for Zig Motivation Zig is a general-purpose programming language designed for robustness, optimality, and maintainability. The programming philosophy of Zig matches up nicely with the programming philosophy of the BEAM VM and in particular its emphasis on simplicity and structure should very appealing to the practitioners of Elixir. The following features make Zig extremely amenable to inline language support in a BEAM language: simplicity. Zig&#39;s syntax is definable in a simple YACC document and Zig takes a stance against making its featureset more complex (though it may evolve somewhat en route to 1.0) Composability. Zig is unopinionated about how to go about memory allocations. Its allocator interface is very easily able to be backed by the BEAM&#39;s, which means that you have access to generic memory allocation strategies through its composable allocator scheme. C integration. It&#39;s very easy to design C-interop between Zig and C. In fact, Zig is likely to be an easier glue language for C ABIs than C. OOM safety and sanity. Zig makes it easy to defer memory destruction, meaning you often don&#39;t have to worry about memory leaks. By design, alloc should be failable (as is the BEAM&#39;s alloc) so it offers an extra layer of protection for you BEAM VM against an OOM situation. Basic usage In the BEAM, you can define a NIF by consulting the following document and implementing the appropriate shared object/DLL callbacks. However, Zigler will take care of all of this for you. Simply use Zigler in your module, providing the app atom in the property list. Then, use the sigil_Z/2 macro and write zig code. Any nifs you define should be preceded with the /// nif: function_name/arity zig docstring. Example defmodule MyModule do use Zigler, otp_app: :my_app ~Z&quot;&quot;&quot; /// nif: my_func/1 fn my_func(val: i64) i64 { return val + 1; } &quot;&quot;&quot; end Zigler will automatically fill out the appropriate NIF C template, compile the shared object, and bind it into the module pre-compilation. In the case of the example, there will be a MyModule.my_func/1 function call found in the module. Zigler will also make sure that your statically-typed Zig data are guarded when you marshal it from the dynamically-typed BEAM world. However, you may only pass in and return certain types (though the generic beam.term type) is supported. Environment Sometimes, you will need to pass the BEAM environment (which is the code execution context, including process info, etc.) into the NIF function. In this case, you should pass it as the first argument, as a beam.env type value. Example defmodule MyModule do use Zigler, otp_app: :my_app ~Z&quot;&quot;&quot; /// nif: my_func_with_env/1 fn my_func_with_env(env: beam.env, pid: beam.pid) void { var sendable_term: []u64 = &quot;ping&quot;[0..]; var msg = beam.make_slice(env, sendable_term); var res = e.enif_send(env, pid, env, msg); } &quot;&quot;&quot; end External Libraries If you need to bind static (*.a) or dynamic (*.so) libraries into your module, you may link them with the :libs argument. Note that for shared libraries, a library with an identical path must exist in the target release environment. Example defmodule Blas do use Zigler, otp_app: :my_app, libs: [&quot;/usr/lib/x86_64-linux-gnu/blas/libblas.so&quot;], include: [&quot;/usr/include/x86_64-linux-gnu&quot;] ~Z&quot;&quot;&quot; const blas = @cImport({ @cInclude(&quot;cblas.h&quot;); ... Compilation assistance If something should go wrong, Zigler will translate the Zig compiler error into an Elixir compiler error, and let you know exactly which line in the ~Z block it came from. Syntactic Sugar Some of the erlang nif terms can get unwieldy, especially in Zig, which prefers terseness. Each of the basic BEAM types is shadowed by a Zig type in the beam module. The beam struct is always imported into the header of the zig file used, so all zig code in the same directory as the module should have access to the beam struct if they @import(&quot;beam.zig&quot;) Importing files If you need to write code outside of the basic module (you will, for anything non-trivial), just place it in the same directory as your module. Example ~Z&quot;&quot;&quot; const extra_code = @import(&quot;extra_code.zig&quot;); /// nif: use_extra_code/1 fn use_extra_code(val: i64) i64 { return extra_code.extra_fn(val); } &quot;&quot;&quot; If you would like to include a custom c header file, create an include/ directory inside your path tree and it will be available to zig as a default search path as follows: ~Z&quot;&quot;&quot; const c = @cImport({ @cInclude(&quot;my_c_header.h&quot;); }); // nif: my_nif/1 ... &quot;&quot;&quot; Documentation Use the builtin zig /// docstring to write your documentation. If it&#39;s in front of the nif declaration, it will wind up in the correct place in your elixir documentation. See Zigler.Doc for more information on how to document in zig and what to document. See Mix.Tasks.ZigDoc for information on how to get your Elixir project to incorporate zig documentation. Tests Use the builtin zig test keyword to write your internal zig unit tests. These can be imported into an ExUnit module by following this example: defmodule MyTest do use ExUnit.Case use Zigler.Unit zigtest ModuleWithZigCode end See Zigler.Unit for more information."},{"ref":"Zigler.html#nif_dir/1","title":"Zigler.nif_dir/1","type":"function","doc":""},{"ref":"Zigler.html#nif_name/2","title":"Zigler.nif_name/2","type":"function","doc":""},{"ref":"Zigler.html#sigil_Z/2","title":"Zigler.sigil_Z/2","type":"macro","doc":"Parses zig code and then accumulates it into the module&#39;s :zigler attribute. Doesn&#39;t actually write any code, since it can all be taken care of in the Zigler.Compiler__before_compile__/1 directive."},{"ref":"Zigler.html#sigil_z/2","title":"Zigler.sigil_z/2","type":"macro","doc":""},{"ref":"Zigler.Assembler.html","title":"Zigler.Assembler","type":"module","doc":"Phase One of Zigler compilation process. Looks through the contents of the zig code and creates a map of required source code files. Then outputs a struct containing the relevant information."},{"ref":"Zigler.Assembler.html#assemble_asset!/2","title":"Zigler.Assembler.assemble_asset!/2","type":"function","doc":""},{"ref":"Zigler.Assembler.html#assemble_assets!/2","title":"Zigler.Assembler.assemble_assets!/2","type":"function","doc":"assembles zig assets, taking them from their source to and putting them into the target directory."},{"ref":"Zigler.Assembler.html#assemble_kernel!/1","title":"Zigler.Assembler.assemble_kernel!/1","type":"function","doc":"assembles the &quot;core&quot; part of the zig adapter code, these are parts for shimming Zig libraries into the BEAM that are going to be @import&#39;d into the runtime by default."},{"ref":"Zigler.Assembler.html#parse_code/2","title":"Zigler.Assembler.parse_code/2","type":"function","doc":""},{"ref":"Zigler.Assembler.html#parse_file/2","title":"Zigler.Assembler.parse_file/2","type":"function","doc":""},{"ref":"Zigler.Assembler.html#t:file_type/0","title":"Zigler.Assembler.file_type/0","type":"type","doc":""},{"ref":"Zigler.Assembler.html#t:t/0","title":"Zigler.Assembler.t/0","type":"type","doc":""},{"ref":"Zigler.Code.html","title":"Zigler.Code","type":"module","doc":"all code responsible for generating zig code lives in this module."},{"ref":"Zigler.Code.html#adapter/1","title":"Zigler.Code.adapter/1","type":"function","doc":""},{"ref":"Zigler.Code.html#aggregate_includes/1","title":"Zigler.Code.aggregate_includes/1","type":"function","doc":""},{"ref":"Zigler.Code.html#c_imports/1","title":"Zigler.Code.c_imports/1","type":"function","doc":""},{"ref":"Zigler.Code.html#footer/1","title":"Zigler.Code.footer/1","type":"function","doc":""},{"ref":"Zigler.Code.html#generate_main/1","title":"Zigler.Code.generate_main/1","type":"function","doc":""},{"ref":"Zigler.Code.html#zig_imports/1","title":"Zigler.Code.zig_imports/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html","title":"Zigler.Code.LongRunning","type":"module","doc":"Generates code for long-running nifs. long-running functions require several parts to get right. a zig struct needs to be that holds arguments relvant to the long-running nif, these arguments need to be cleaned up in a sane fashion once the nif has completed running. This struct is going to be argetric on the nif arguments, and will be packed into a BEAM resource. a packer function which takes the beam arguments and shoves them into a zig struct to be stored in a BEAM resource, then launches the function, returning the resource. a launcher function which runs the packer wrapping the errors from the launch function. The launch function must be a nif function, as it will be called from the BEAM. a harness function which is passed the resource struct, and is responsible for updating the resource, sending back a &quot;finished&quot; message to parent context. The harness function will be responsible for wrapping the declared nif function. a fetching function which is passed the resource struct, and marshals it back into BEAM terms, returning the result. This function must be a nif function, as it will be called from the BEAM. a cleanup function which is responsible for cleaning up the resource object once the thread has completed its task."},{"ref":"Zigler.Code.LongRunning.html#adapter/1","title":"Zigler.Code.LongRunning.adapter/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache/1","title":"Zigler.Code.LongRunning.cache/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache_cleanup/1","title":"Zigler.Code.LongRunning.cache_cleanup/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache_ptr/1","title":"Zigler.Code.LongRunning.cache_ptr/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#cache_struct/1","title":"Zigler.Code.LongRunning.cache_struct/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#fetcher/1","title":"Zigler.Code.LongRunning.fetcher/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#fetcher_fn/1","title":"Zigler.Code.LongRunning.fetcher_fn/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#function_skeleton/1","title":"Zigler.Code.LongRunning.function_skeleton/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#harness/1","title":"Zigler.Code.LongRunning.harness/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#harness_fn/1","title":"Zigler.Code.LongRunning.harness_fn/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#launcher/1","title":"Zigler.Code.LongRunning.launcher/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#launcher_fn/1","title":"Zigler.Code.LongRunning.launcher_fn/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#packer/1","title":"Zigler.Code.LongRunning.packer/1","type":"function","doc":""},{"ref":"Zigler.Code.LongRunning.html#packer_fn/1","title":"Zigler.Code.LongRunning.packer_fn/1","type":"function","doc":""},{"ref":"Zigler.Doc.html","title":"Zigler.Doc","type":"module","doc":"Translates the docstrings from your Zig code into Elixir documentation. For instructions on how to incorporate this into an Elixir project, consult Mix.Tasks.ZigDoc Documentation forms Currently, Zigler recognizes four types of code segments which should be documented. functions types values errors Functions functions have the following signature: pub fn &lt;identifier&gt;(&lt;arguments&gt;) &lt;type&gt; { and may have the property of being comptime which is due to either the function being itself a comptime function or it having a comptime argument. NB only pub functions are documented in Zigler, following the Elixir philosophy that only public functions should be documented. Types types have the following signature: pub const &lt;identifier&gt;=&lt;value&gt;; Values values can have one of the following signatures: pub const &lt;identifier&gt;=&lt;value&gt;; pub var &lt;identifier&gt;=&lt;value&gt;; Note that the constant value form is indistinguishable from the type form without doing a full parse and evaluation of the Zig code. In order to avoid doing this, to disambiguate between the two, you must prepend constant value docststrings with a !value token. Example /// !value /// /// a constant representing the value 47. pub const fortyseven = 47; Errors errors appear inside special error struct and should be documented at the per-value level: Example /// docstring for this error struct (if desired) pub const my_error = error { /// docstring for ErrorEnum1 ErrorEnum1, /// docstring for ErrorEnum2 ErrorEnum2 } Scope This module will generate documentation from all Zig code that resides in the same code directory as the base module (or overridden directory, if applicable). Zig code in subdirectories will not be subjected to document generation."},{"ref":"Zigler.Module.html","title":"Zigler.Module","type":"module","doc":"this struct represents all information a zigler module bound to a nif should have. WIP."},{"ref":"Zigler.Module.html#imports/1","title":"Zigler.Module.imports/1","type":"function","doc":""},{"ref":"Zigler.Module.html#t:t/0","title":"Zigler.Module.t/0","type":"type","doc":""},{"ref":"Zigler.Parser.Nif.html","title":"Zigler.Parser.Nif","type":"module","doc":"This datastructure represents structured information about a single nif inside of a Zigler.sigil_Z/2 block. This is used to generate the exported_nifs variable which is an array of ErlNifFunc structs. The following keys are implemented: name: (atom/0) the function name to be bound into the module arity: (arity/0) the arity of the erlang function (the zig function may have a different arity). doc: (iodata/0) zig docstrings which should be turned into elixir docs args: (String.t/0) a list of zig types which are the arguments for the function retval: (String.t/0) the type of the return value opts: (t:keyword) list of nif options. long: true -- if the nif should run in a separate OS thread. dirty: :cpu -- if the nif should run in a dirty cpu scheduler. dirty: :io -- if the nif should run in a dirty io scheduler."},{"ref":"Zigler.Parser.Nif.html#register_function_header/2","title":"Zigler.Parser.Nif.register_function_header/2","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#validate_args/3","title":"Zigler.Parser.Nif.validate_args/3","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#validate_arity/3","title":"Zigler.Parser.Nif.validate_arity/3","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#validate_retval/3","title":"Zigler.Parser.Nif.validate_retval/3","type":"function","doc":""},{"ref":"Zigler.Parser.Nif.html#t:option/0","title":"Zigler.Parser.Nif.option/0","type":"type","doc":""},{"ref":"Zigler.Parser.Nif.html#t:t/0","title":"Zigler.Parser.Nif.t/0","type":"type","doc":""},{"ref":"Zigler.Parser.Resource.html","title":"Zigler.Parser.Resource","type":"module","doc":"resource parser object"},{"ref":"Zigler.Parser.Resource.html#register_resource_definition/1","title":"Zigler.Parser.Resource.register_resource_definition/1","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html","title":"Zigler.Parser.ResourceCleanup","type":"module","doc":"resource cleanup struct"},{"ref":"Zigler.Parser.ResourceCleanup.html#register_function_header/2","title":"Zigler.Parser.ResourceCleanup.register_function_header/2","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html#validate_args/3","title":"Zigler.Parser.ResourceCleanup.validate_args/3","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html#validate_arity/3","title":"Zigler.Parser.ResourceCleanup.validate_arity/3","type":"function","doc":""},{"ref":"Zigler.Parser.ResourceCleanup.html#validate_retval/3","title":"Zigler.Parser.ResourceCleanup.validate_retval/3","type":"function","doc":""},{"ref":"Zigler.Parser.Unit.html","title":"Zigler.Parser.Unit","type":"module","doc":"parses zig code and converts test blocks to test functions"},{"ref":"Zigler.Parser.Unit.html#parse/2","title":"Zigler.Parser.Unit.parse/2","type":"function","doc":""},{"ref":"Zigler.Parser.Unit.html#unit_parser/2","title":"Zigler.Parser.Unit.unit_parser/2","type":"function","doc":"Parses the given binary as unit_parser. Returns {:ok, [token], rest, context, position, byte_offset} or {:error, reason, rest, context, line, byte_offset} where position describes the location of the unit_parser (start position) as {line, column_on_line}. Options :line - the initial line, defaults to 1 :byte_offset - the initial byte offset, defaults to 0 :context - the initial context value. It will be converted to a map"},{"ref":"Zigler.Patches.html","title":"Zigler.Patches","type":"module","doc":"occasionally zig and/or the llvm bits that get pulled down have errors. This module actively patches code when this situation occurs."},{"ref":"Zigler.Patches.html#patch_needed?/2","title":"Zigler.Patches.patch_needed?/2","type":"function","doc":""},{"ref":"Zigler.Patches.html#sync/1","title":"Zigler.Patches.sync/1","type":"function","doc":""},{"ref":"Zigler.Unit.html","title":"Zigler.Unit","type":"module","doc":"Hooks your zig code into ExUnit, by converting zig tests into ExUnit tests. Usage Example Inside your zig code (dependent.zig): const beam = @import(&quot;beam.zig&quot;); const assert = beam.assert; fn one() i64 { return 1; } test &quot;the one function returns one&quot; { assert(one() == 1); } Inside your elixir code: defmodule MyZigModule do use Zigler, otp_app: :my_app ~Z&quot;&quot;&quot; const dependent = @import(&quot;dependent.zig&quot;); /// nif: some_nif_fn/1 ... &quot;&quot;&quot; end Inside your test module: defmodule MyZigTest do use ExUnit.Case, async: true use Zigler.Unit zigtest MyZigModule end Scope This module will run tests in all zig code that resides in the same code directory as the base module (or overridden directory, if applicable). Zig code in subdirectories will not be subjected to test conversion, so if you would like to run a subset of tests using the Zig test facility (and without the support of a BEAM VM), you should put them in subdirectories."},{"ref":"Zigler.Unit.html#zigtest/2","title":"Zigler.Unit.zigtest/2","type":"macro","doc":"loads a module that wraps a zigler NIF, consults the corresponding zig code, generating the corresponding zig tests. Must be called from within a module that has use ExUnit.Case and use Zigler"},{"ref":"readme.html","title":"Zigler","type":"extras","doc":"Zigler Library test status: Dependent package test status:"},{"ref":"readme.html#installation","title":"Zigler - Installation","type":"extras","doc":"Zigler is available in Hex, and the package can be installed by adding zigler to your list of dependencies in mix.exs: def deps do [ {:zigler, &quot;~&gt; 0.3.0&quot;, runtime: false} ] end"},{"ref":"readme.html#documentation","title":"Zigler - Documentation","type":"extras","doc":"Docs can be found at https://hexdocs.pm/zigler."},{"ref":"readme.html#currently-supported-platforms","title":"Zigler - Currently supported platforms","type":"extras","doc":"Linux FreeBSD (tested, but not subjected to CI) MacOS (I believe it works but is still offically untested)"},{"ref":"readme.html#zig-nifs-made-easy","title":"Zigler - Zig Nifs made easy","type":"extras","doc":"Wouldn&#39;t it be nice if you could make NIFs as easily as you can use the asm keyword in C? This is now possible, using the magic of Zig. defmodule ExampleZig do use Zigler, otp_app: :my_app ~Z&quot;&quot;&quot; /// nif: example_fun/2 fn example_fun(value1: f64, value2: f64) bool { return value1 &gt; value2; } &quot;&quot;&quot; end iex&gt; ExampleZig.example_fun(0.1, 0.4) false iex&gt; ExampleZig.example_fun(0.8, -0.8) true Zigler will do automatic type marshalling between Elixir code and Zig code. It will also convert trickier types into types you care about, for example: defmodule ZigCollections do use Zigler, otp_app: :my_app ~Z&quot;&quot;&quot; /// nif: string_count/1 fn string_count(string: []u8) i64 { return @intCast(i64, string.len); } /// nif: list_sum/1 fn list_sum(array: []f64) f64 { var sum: f64 = 0.0; for(array) | item | { sum += item; } return sum; } &quot;&quot;&quot; end iex&gt; ZigCollections.string_count(&quot;hello zig&quot;) 9 iex&gt; ZigCollections.list_sum([1.0, 2.0, 3.0]) 6.0 Memory allocation with zigler is easy! A standard BEAM allocator is provided for you, so any zig code you import will play nice with the BEAM. defmodule Allocations do use Zigler, otp_app: :my_app ~Z&quot;&quot;&quot; /// nif: double_atom/1 fn double_atom(env: beam.env, string: []u8) beam.atom { var double_string = beam.allocator.alloc(u8, string.len * 2) catch { return beam.raise_enomem(env); }; defer beam.allocator.free(double_string); for (string) | char, i | { double_string[i] = char; double_string[i + string.len] = char; } return beam.make_atom(env, double_string); } &quot;&quot;&quot; end iex&gt; Allocations.double_atom(&quot;foo&quot;) :foofoo It is a goal for Zigler to make using it to bind C libraries easier than using C to bind C libraries. Here is an example: defmodule BlasDynamic do use Zigler, otp_app: :zigler, libs: [&quot;/usr/lib/x86_64-linux-gnu/blas/libblas.so&quot;], include: [&quot;/usr/include/x86_64-linux-gnu&quot;] ~Z&quot;&quot;&quot; const blas = @cImport({ @cInclude(&quot;cblas.h&quot;); }); /// nif: blas_axpy/3 fn blas_axpy(env: beam.env, a: f64, x: []f64, y: []f64) beam.term { if (x.len != y.len) { return beam.raise_function_clause_error(env); } blas.cblas_daxpy(@intCast(c_int, x.len), a, x.ptr, 1, y.ptr, 1); return beam.make_f64_list(env, y) catch { return beam.raise_function_clause_error(env); }; } &quot;&quot;&quot; end test &quot;we can use dynamically-linked blas&quot; do # returns aX+Y assert [11.0, 18.0] == BlasDynamic.blas_axpy(3.0, [2.0, 4.0], [5.0, 6.0]) end Zigler even has support for zig docstrings. defmodule AllTheDocs do use Zigler, otp_app: :zigler ~Z&quot;&quot;&quot; /// a zero-arity function which returns 47. /// nif: zero_arity/0 fn zeroarity() i64 { return 47; } &quot;&quot;&quot; end iex&gt; h AllTheDocs.zeroarity def zeroarity() a zero-arity function which returns 47."},{"ref":"readme.html#zigler-principles","title":"Zigler - Zigler Principles","type":"extras","doc":"Make doing the right thing easy. Use magic, but sparingly. Let the user see behind the curtain. Let the user dial in magic as they choose. Magic shouldn&#39;t get in the way"}]